<?php
/**
 * Created by PhpStorm.
 * User: Noa
 * Date: 02/07/2016
 * Time: 00:32
 */

namespace Noa\PhpRouter;

class Route {

    /**
     * A Route instance
     * @var Route $instance
     */
    protected static $instance = null;

    protected $parameters = array();

    protected $controller = null;

    protected $controllerRootPath = null;

    protected $verb = "GET";

    protected $route = null;


    /**
     * Init values
     * @param $route
     * @param $controller
     */
    protected function init($route, $controller) {
        $this->route = $route;
        $this->controller = $controller;
    }

    /**
     * Allow to set HTTP verb
     * @param $verb
     */
    protected function setVerb($verb){
        $this->verb = $verb;
    }

    /**
     * Return user defined parameters regexes
     * @return array Associative array of parameters
     */
    protected function getParameters() {
        return $this->parameters;
    }

    /**
     * Allow to append parameters
     * @param $name
     * @param $regex
     * @param null $default
     */
    protected function addParameters($name, $regex, $default = null) {
        $this->parameters["{".$name."}"] = $regex;
    }


    /**
     * Retrieve a Route singleton
     * @return Route
     */
    protected static function getInstance() {

        if (is_null(self::$instance)) {
            self::$instance = new Route();
        }

        return self::$instance;
    }

    /**
     * Get specific controller function
     * @param string $controller Path to controller
     * @return void
     */
    protected function parseController($controller, $parameters) {

    }

    /**
     * Find parameters from string route
     * return string Parsed parameters
     */
    protected function parseRoute() {

        $parameters = $this->getParameters();

        $parsedRoute = preg_replace_callback("/{(\w*)}/u", function($match) use ($parameters) {
            if(isset($parameters[$match[0]])){
                return ("(".$parameters[$match[0]].")");

            } else{
                return("(.+)");

            }
        },  $this->route);

        return str_replace('/', '\/', $parsedRoute);

    }

    /**
     * Define a get route and call controller function
     * @param string $route String route (ex: test/{id}/{name} )
     * @param string $controller String path controller (ex: Package.SubPackage@method )
     * @return Route
     */
    public static function get($route, $controller) {

        $instance = Route::getInstance();
        $instance->setVerb("GET");
        $instance->init($route, $controller);

        return $instance;

    }

    /**
     * Define a post route and call controller function
     * @param string $route String route (ex: test/{id}/{name} )
     * @param string $controller String path controller (ex: Package.SubPackage@method )
     * @return Route
     */
    public static function post($route, $controller) {

        $instance = Route::getInstance();
        $instance->setVerb("POST");
        $instance->init($route, $controller);

        return $instance;

    }

    public static function setControllerRootPath($controllerRootPath) {
        $instance = Route::getInstance();
        $instance->controllerRootPath = $controllerRootPath;
    }

    public function where($name, $regex, $default=null) {
        $instance = Route::getInstance();

        $instance->addParameters($name, $regex, $default);

        return $instance;

    }

    /**
     * Save a route into router
     * @param string $name Optional route name
     */
    public function save($name=null) {

        $instance = Route::getInstance();
        echo $instance->parseRoute();
        echo $instance->parseController();

    }

}
